==> omnibuild-0.3.0.tar.gz: lib/src/lambdas.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

typedef String _TemplateLambda(LambdaContext ctx);

/// Helper lambda's for the omnibuild template.
Map<String, _TemplateLambda> _fnLambdas = {
  /// Remove terminating line feed.
  'rmLf': (LambdaContext ctx) {
    var str = ctx.renderString();
    if (str.isNotEmpty && str[str.length - 1] == '\n') {
      return str.substring(0, str.length - 1);
    } else {
      return str;
    }
  },

  /// Indent with two spaces.
  'indent': (LambdaContext ctx) => _prefixEachLine(ctx.renderString(), '  '),

  /// Comment each line with two slashes.
  'normalComment': (LambdaContext ctx) =>
      _prefixEachLine(ctx.renderString(), '// '),

  /// Comment each line with three slashes.
  'docComment': (LambdaContext ctx) =>
      _prefixEachLine(ctx.renderString(), '/// '),

  /// Find type that should be used by the PDS type that can be asserted.
  /// Note that Dart cannot assert generic class type parameters, so a List<int>
  /// must be asserted as List and reconstructed as List<int>.
  'pdsCast': (LambdaContext ctx) {
    String type = ctx.lookup('pdsType');
    String name = ctx.lookup('name');

    // If this is an enumeration we can already return the result.
    if (ctx.lookup('enumeration') == true) {
      return " && pds['$name'] is int";
    }

    // If the pdsType is empty, we'll fallback to the actual type.
    if (type.isEmpty) {
      type = ctx.lookup('type');
    }

    // Remove everything after the first '<' to get the base type.
    var idx = type.indexOf('<');
    type = idx != -1 ? type.substring(0, idx) : type;

    // Dynamic types are not asserted.
    if (type == 'dynamic') {
      type = '';
    }

    // Only return `' && pds[$name] is $type'` if type is not empty.
    return type.isNotEmpty ? " && pds['$name'] is $type" : '';
  },

  /// Get arguments that are defined in a message constructor.
  'constructorArgs': (LambdaContext ctx) => _constructorArgs(
      ctx.lookup('properties') as List<OmnibuildProperty>, true),

  /// Get arguments that are passed to the constructor in the .fromPds factory.
  'constructorCall': (LambdaContext ctx) => _constructorArgs(
      ctx.lookup('properties') as List<OmnibuildProperty>, false),

  /// Construct value from PDS data
  'reverseConstruct': (LambdaContext ctx) {
    String access = ctx.renderString();
    String type = ctx.lookup('type');

    if (ctx.lookup('enumeration') == true) {
      return "$type.values[$access]";
    } else {
      return getPdsToTypeCode(
          type, access, ctx.lookup('pdsType') as String, false);
    }
  },

  /// Generate map that is used to convert fields to PDS.
  'toPdsMap': (LambdaContext ctx) {
    var pdsMap = [];
    List<OmnibuildProperty> properties =
        ctx.lookup('properties') as List<OmnibuildProperty>;

    // Read properties.
    properties.forEach((OmnibuildProperty property) {
      if (property.enumeration) {
        pdsMap.add("'${property.name}': ${property.name}.index");
      } else {
        pdsMap.add(
            "'${property.name}': ${getTypeToPdsCode(property.type, '${property.name}')}");
      }
    });

    return "{${pdsMap.join(',')}}";
  },

  /// Generate map that contains the funnel values from a message.
  'getFunnelMap': (LambdaContext ctx) {
    var map = new List<String>();
    Map<String, OmnibuildFunnelValue> funnel =
        ctx.lookup('funnel') as Map<String, OmnibuildFunnelValue>;
    funnel.forEach(
        (String key, OmnibuildFunnelValue v) => map.add("'$key': ${v.code}"));
    return "{${map.join(',')}}";
  },

  /// Get arguments for the message handler constructor.
  /// Note: mind the starting comma!
  'handlerConstructArgs': (LambdaContext ctx) {
    Map<String, OmnibuildFunnelValue> funnel =
        ctx.lookup('funnel') as Map<String, OmnibuildFunnelValue>;
    var args = new List<String>();

    // Read properties.
    funnel.forEach((String key, OmnibuildFunnelValue v) {
      args.add("${v.type} $key");
    });

    return args.isNotEmpty ? ',{${args.join(',')}}' : '';
  },

  /// Get arguments for the message handler constructor funnel map.
  /// Note: mind the starting comma!
  'handlerConstructorMap': (LambdaContext ctx) {
    Map<String, OmnibuildFunnelValue> funnel =
        ctx.lookup('funnel') as Map<String, OmnibuildFunnelValue>;
    var args = new List<String>();

    // Read properties.
    funnel.forEach((String key, _) {
      args.add("'$key': $key");
    });

    return args.isNotEmpty ? ',{${args.join(',')}}' : '';
  },

  /// Returning type for the message handler callback.
  'handlerCbReturn': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusEvent'
        ? 'Future<Null>'
        : 'Future<OmnibusResponse>';
  },

  /// Only return inner content when the message is an event or request.
  'onlyIfEventOrRequest': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusResponse'
        ? ''
        : ctx.renderString();
  },

  /// Only return inner content when the message is a response.
  'onlyIfResponse': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusResponse'
        ? ctx.renderString()
        : '';
  }
};

/// Generate Dart code for the OmnibusMessage constructor. If [define] is
/// true the arguments will be generated as constructor parameters, else the
/// arguments will be generated as constructor input.
String _constructorArgs(List<OmnibuildProperty> properties, bool define) {
  var positionalParameters = [];
  var optPositionalParameters = [];
  var namedParameters = [];

  // Read properties.
  properties.forEach((OmnibuildProperty property) {
    /// Add property to a parameter group.
    if (property.named) {
      // Add parameter to named parameters.
      namedParameters.add(define
          ? 'this.${property.name}: ${property.defaultValue}'
          : '${property.name}: ${property.name}');
    } else {
      // Add parameter to optional positional parameters if a default value is
      // defined.
      if (property.defaultValue.isEmpty) {
        positionalParameters
            .add(define ? 'this.${property.name}' : '${property.name}');
      } else {
        optPositionalParameters.add(define
            ? 'this.${property.name} = ${property.defaultValue}'
            : '${property.name}');
      }
    }
  });

  // Generate parameters string.
  var parameters = positionalParameters.join(',');
  if (optPositionalParameters.isNotEmpty) {
    parameters = (parameters.isNotEmpty ? parameters + ',' : '') +
        (define
            ? '[${optPositionalParameters.join(',')}]'
            : '${optPositionalParameters.join(',')}');
  }
  if (namedParameters.isNotEmpty) {
    parameters = (parameters.isNotEmpty ? parameters + ',' : '') +
        (define
            ? '{${namedParameters.join(',')}}'
            : '${namedParameters.join(',')}');
  }

  return parameters;
}

==> omnibuild-0.4.0.tar.gz: lib/src/lambdas.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

typedef String _TemplateLambda(LambdaContext ctx);

/// Helper lambda's for the omnibuild template.
Map<String, _TemplateLambda> _fnLambdas = {
  /// Remove terminating line feed.
  'rmLf': (LambdaContext ctx) {
    var str = ctx.renderString();
    if (str.isNotEmpty && str[str.length - 1] == '\n') {
      return str.substring(0, str.length - 1);
    } else {
      return str;
    }
  },

  /// Indent with two spaces.
  'indent': (LambdaContext ctx) => _prefixEachLine(ctx.renderString(), '  '),

  /// Comment each line with two slashes.
  'normalComment': (LambdaContext ctx) =>
      _prefixEachLine(ctx.renderString(), '// '),

  /// Comment each line with three slashes.
  'docComment': (LambdaContext ctx) =>
      _prefixEachLine(ctx.renderString(), '/// '),

  /// Find type that should be used by the PDS type that can be asserted.
  /// Note that Dart cannot assert generic class type parameters, so a List<int>
  /// must be asserted as List and reconstructed as List<int>.
  'pdsCast': (LambdaContext ctx) {
    String type = ctx.lookup('pdsType');
    String name = ctx.lookup('name');

    // If this is an enumeration we can already return the result.
    if (ctx.lookup('enumeration') == true) {
      return " && pds['$name'] is int";
    }

    // If the pdsType is empty, we'll fallback to the actual type.
    if (type.isEmpty) {
      type = ctx.lookup('type');
    }

    // Remove everything after the first '<' to get the base type.
    var idx = type.indexOf('<');
    type = idx != -1 ? type.substring(0, idx) : type;

    // Dynamic types are not asserted.
    if (type == 'dynamic') {
      type = '';
    }

    // Only return `' && pds[$name] is $type'` if type is not empty.
    return type.isNotEmpty ? " && pds['$name'] is $type" : '';
  },

  /// Get arguments that are defined in a message constructor.
  'constructorArgs': (LambdaContext ctx) => _constructorArgs(
      ctx.lookup('properties') as List<OmnibuildProperty>, true),

  /// Get arguments that are passed to the constructor in the .fromPds factory.
  'constructorCall': (LambdaContext ctx) => _constructorArgs(
      ctx.lookup('properties') as List<OmnibuildProperty>, false),

  /// Construct value from PDS data
  'reverseConstruct': (LambdaContext ctx) {
    String access = ctx.renderString();
    String type = ctx.lookup('type');

    if (ctx.lookup('enumeration') == true) {
      return "$type.values[$access]";
    } else {
      return getPdsToTypeCode(
          type, access, ctx.lookup('pdsType') as String, false);
    }
  },

  /// Generate map that is used to convert fields to PDS.
  'toPdsMap': (LambdaContext ctx) {
    var pdsMap = [];
    List<OmnibuildProperty> properties =
        ctx.lookup('properties') as List<OmnibuildProperty>;

    // Read properties.
    properties.forEach((OmnibuildProperty property) {
      if (property.enumeration) {
        pdsMap.add("'${property.name}': ${property.name}.index");
      } else {
        pdsMap.add(
            "'${property.name}': ${getTypeToPdsCode(property.type, '${property.name}')}");
      }
    });

    return "{${pdsMap.join(',')}}";
  },

  /// Get arguments for the message handler constructor.
  /// Note: mind the starting comma!
  'handlerConstructArgs': (LambdaContext ctx) {
    List<OmnibuildProperty> filter =
        ctx.lookup('filter') as List<OmnibuildProperty>;
    var args = new List<String>();

    // Read filter.
    filter.forEach((OmnibuildProperty property) {
      args.add("this.${property.name}: null");
    });

    return args.isNotEmpty ? ',{${args.join(',')}}' : '';
  },

  /// Returning type for the message handler callback.
  'handlerCbReturn': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusEvent'
        ? 'Future<Null>'
        : 'Future<OmnibusResponse>';
  },

  'handlerRunCb': (LambdaContext ctx) {
    List<OmnibuildProperty> filter =
        ctx.lookup('filter') as List<OmnibuildProperty>;
    var conditions = new List<String>();

    // Read filter.
    filter.forEach((OmnibuildProperty property) {
      conditions.add(
          "(${property.name} == null || ${property.name} == message.${property.name})");
    });

    // Return code.
    if (conditions.isNotEmpty) {
      return 'if (${conditions.join(' && ')}) { return [_callback(message)]; } else { return []; }';
    } else {
      return 'return [_callback(message)];';
    }
  },

  /// Only return inner content when the message is an event or request.
  'onlyIfEventOrRequest': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusResponse'
        ? ''
        : ctx.renderString();
  },

  /// Only return inner content when the message is a response.
  'onlyIfResponse': (LambdaContext ctx) {
    return ctx.lookup('superClass') == 'OmnibusResponse'
        ? ctx.renderString()
        : '';
  }
};

/// Generate Dart code for the OmnibusMessage constructor. If [define] is
/// true the arguments will be generated as constructor parameters, else the
/// arguments will be generated as constructor input.
String _constructorArgs(List<OmnibuildProperty> properties, bool define) {
  var positionalParameters = [];
  var optPositionalParameters = [];
  var namedParameters = [];

  // Read properties.
  properties.forEach((OmnibuildProperty property) {
    /// Add property to a parameter group.
    if (property.named) {
      // Add parameter to named parameters.
      namedParameters.add(define
          ? 'this.${property.name}: ${property.defaultValue}'
          : '${property.name}: ${property.name}');
    } else {
      // Add parameter to optional positional parameters if a default value is
      // defined.
      if (property.defaultValue.isEmpty) {
        positionalParameters
            .add(define ? 'this.${property.name}' : '${property.name}');
      } else {
        optPositionalParameters.add(define
            ? 'this.${property.name} = ${property.defaultValue}'
            : '${property.name}');
      }
    }
  });

  // Generate parameters string.
  var parameters = positionalParameters.join(',');
  if (optPositionalParameters.isNotEmpty) {
    parameters = (parameters.isNotEmpty ? parameters + ',' : '') +
        (define
            ? '[${optPositionalParameters.join(',')}]'
            : '${optPositionalParameters.join(',')}');
  }
  if (namedParameters.isNotEmpty) {
    parameters = (parameters.isNotEmpty ? parameters + ',' : '') +
        (define
            ? '{${namedParameters.join(',')}}'
            : '${namedParameters.join(',')}');
  }

  return parameters;
}
